module HiggsVar
  use Consts
  use QCD
  implicit none

  integer :: higgs_order, sub_order
  real(8) :: prefactor, prefactor2, mt, mH, Ecms, tau, eta, Lt, Ls
  real(8) :: mbmb, mb

  real(8) :: gammaCuspA(0:3), gamma_V(0:2), gamma_VH(0:2), gamma_phi(0:2)
  
  real(8), external :: ddilog, dgammf, dpsipg
  complex(8), external :: wgplg

  real(8) :: mu_t, mu_h, mu_s, alphas_mut, alphas_muh, alphas_mus
  complex(8) :: alphas_imuh

  real(8) :: GFew

  real(8) :: mut0, muh0, mus0, muf0

  real(8) :: iERR = 1d-3

contains

  subroutine Higgs_UpdateCoeff
    implicit none
    gammaCuspA(0) = 12d0
    gammaCuspA(1) = 149.5647d0 - 13.33333d0 * nf
    gammaCuspA(2) = 2643.521d0 - 412.1717d0 * nf - 1.777778d0 * nf**2
    gammaCuspA(3) = gammaCuspA(2)**2 / gammaCuspA(1)
    gamma_VH(0) = -22d0 + 1.333333d0 * nf
    gamma_VH(1) = -309.4936d0 + 27.19804d0 * nf
    gamma_VH(2) = -6394.243d0 + 858.1089d0 * nf - 12.01437d0 * nf**2
    gamma_V(0) = gamma_VH(0) + 2d0 * beta(0)
    gamma_V(1) = gamma_VH(1) + 4d0 * beta(1)
    gamma_V(2) = gamma_VH(2) + 6d0 * beta(2)
    gamma_phi(0) = 11d0 - 0.6666667d0 * nf
    gamma_phi(1) = 225.8221d0 - 10.66667d0 * nf
    gamma_phi(2) = 4425.448d0 - 528.7195d0 * nf + 6.462963d0 * nf**2
  end subroutine Higgs_UpdateCoeff

  function A(x)
    implicit none
    real(8), intent(in) :: x
    complex(8) :: A
    complex(8) :: f
    if (x >= 1d0) then
       f = asin(1d0/sqrt(x))**2
    else
       f = -(log((1d0+sqrt(1d0-x))/(1d0-sqrt(1d0-x))) - (0d0,pi))**2 / 4d0
    endif
    A = (1d0 + (1d0 - x) * f) * x * 3d0 / 2d0
  end function A

  function Quark_Mass_MSbar(scale, scale0, mass0) result (mass)
    implicit none
    real(8), intent(in) :: scale, scale0, mass0
    real(8) :: mass
    real(8) :: gamma_m(0:1)
    real(8) :: a, a0
    gamma_m(0) = 8d0
    gamma_m(1) = 134.6667d0 - 4.444444d0 * nf
    a = Get_Alphas(scale)/4d0/pi
    a0 = Get_Alphas(scale0)/4d0/pi
    mass = mass0 * (a/a0)**(gamma_m(0)/2d0/beta(0)) * &
         ( 1d0 + ( gamma_m(1)/2d0/beta(0) - beta(1)*gamma_m(0)/2d0/beta(0)**2 ) * (a - a0) )
  end function Quark_Mass_MSbar

  function Alphas_Lambda_Complex(scale2) result (alphas)
    implicit none
    complex(8), intent(in) :: scale2
    complex(8) :: alphas
    complex(8) :: L, LL
    L = log(scale2 / Lambda_QCD**2)
    LL = log(L)
    alphas = 1d0
    if (alphas_order > 0) then
       alphas = alphas - beta(1)/beta(0)**2 * LL/L
    endif
    if (alphas_order > 1) then
       alphas = alphas + beta(1)**2/beta(0)**4/L**2 * &
            ( (LL - .5d0)**2 + beta(2)*beta(0)/beta(1)**2  - 5d0/4d0 )
    endif
    if (alphas_order > 2) then
       alphas = alphas + beta(1)**3/beta(0)**6/L**3 * &
            ( -LL**3 + 5d0*LL**2/2d0 + (2d0 - 3d0*beta(0)*beta(2)/beta(1)**2)*LL &
            + beta(0)**2*beta(3)/2d0/beta(1)**3 - .5d0 )
    endif
    alphas = alphas * 4d0*pi/beta(0)/L
  end function Alphas_Lambda_Complex

end module HiggsVar
