#! /usr/bin/env python

import sys
import os
import math

config = {}

################################################################################

config["mt"] = "173.1d0"
config["mbmb"] = "4.2d0"
config["GFew"] = "1.16208d-5"

config["with_asmz_error"] = ".true."
config["muf"] = "-1d0"

################################################################################
######   Do not touch the following unless you know what you are doing    ######
################################################################################

# For NNLO
config["order"] = 2
pdfsets = {"MSTW" : ("MSTW2008nnlo90cl.LHgrid", "0.11707d0", 2),
           "CTEQ" : ("cteq66.LHgrid", "0.118d0", 1),
           "CT10" : ("CT10.LHgrid", "0.118d0", 1),
           "NNPDF" : ("NNPDF20_100.LHgrid", "0.119d0", 1)}
pdfsets_asmz = {"MSTW" : ( ("MSTW2008nnlo90cl_asmz-90cl.LHgrid", "0.11365d0"),
                           ("MSTW2008nnlo90cl_asmz+90cl.LHgrid", "0.12044d0") ),
                "CTEQ" : ( ("cteq66alphas.LHgrid", "0.116d0"),
                           ("cteq66alphas.LHgrid", "0.120d0") ),
                "CT10" : ( ("CT10as.LHgrid", "0.116d0"),
                           ("CT10as.LHgrid", "0.120d0") ),
                "NNPDF" : ( ("NNPDF20_as_0117_100.LHgrid", "0.117d0"),
                            ("NNPDF20_as_0121_100.LHgrid", "0.121d0") )}

################################################################################
# Note that there is no alpha_s variation in LO sets!
# For LO
#config["order"] = 0
#pdfsets["MSTW"] = ("MSTW2008lo90cl.LHgrid", "0.13939d0", 0)
#pdfsets_asmz["MSTW"] = ()

# For NLO
#config["order"] = 1
#pdfsets["MSTW"] = ("MSTW2008nlo90cl.LHgrid", "0.12018d0", 1)
#pdfsets_asmz["MSTW"] = ( ("MSTW2008nlo90cl_asmz-90cl.LHgrid", "0.11632d0"),
#                         ("MSTW2008nlo90cl_asmz+90cl.LHgrid", "0.12335d0") )

################################################################################

order_string = ["LO", "NLO", "NNLO"]
config["suborder"] = 3
config["exefile"] = "./RunHiggs"

def PrepareConfigFile(pdfset_asmz):
    f = open("RunHiggs.in", "w")

    if config["energy"] == 1960:
        config["collider"] = "Tevatron"
        config["hadron2"] = -1
    else:
        config["collider"] = "LHC"
        config["hadron2"] = 1

    string = "%(pdfset)s\n" + "%(alphasMZ)s\n" + "%(alorder)d\n" \
        + "%(energy)fd0\n" + "%(hadron2)d\n" + "%(mH)fd0\n" + "%(muf)s\n" \
        + "%(order)d\n" + "%(suborder)d\n" \
        + "%(mt)s\n" + "%(mbmb)s\n" + "%(GFew)s\n" \
        + "%(with_asmz_error)s\n"

    if config["with_asmz_error"] == ".true.":
        string += "%d\n" % (len(pdfset_asmz),)
        for a in pdfset_asmz:
            string += "%s\n%s\n" % a

    f.write(string % config)
    f.close()

################################################################################

def usage(cmd):
    print "Usage: %s sqrtS m_H pdfset" % cmd
    print """
Supported PDF sets are (through LHAPDF):
                       MSTW --- MSTW2008NNLO
                       CTEQ --- CTEQ6.6
                       CT10 --- CT10
                      NNPDF --- NNPDF2.0
"""

################################################################################

if __name__ == '__main__':
    try:
        config["energy"] = float(sys.argv[1])
        config["mH"] = float(sys.argv[2])
    except (IndexError, ValueError):
        usage(sys.argv[0])
    else:
        if len(sys.argv) > 3:
            pdfset = sys.argv[3]
        else:
            pdfset = "MSTW"
        if config["mH"] < 100. or config["mH"] > 300.:
            print "Higgs mass excluded!"
        elif config["energy"] < 1960.:
            print "Collider energy too low!"
        elif pdfset not in pdfsets:
            print "PDF set not supported!"
        else:
            config["pdfset"] = pdfsets[pdfset][0]
            config["alphasMZ"] = pdfsets[pdfset][1]
            config["alorder"] = pdfsets[pdfset][2]
            config["order_string"] = order_string[config["order"]]
            PrepareConfigFile(pdfsets_asmz[pdfset])
            print "************************************************************"
            print "  Higgs production at the %(collider)s in %(order_string)s RG-improved QCD" \
                % config
            print "************************************************************\n"
            print " Using " + pdfset + " PDF sets\n"
            os.system("%(exefile)s" % config)

